@@ -11,8 +11,18 @@ import socket
 from smb.SMBConnection import SMBConnection
 from smb.smb_structs import OperationFailure
 
-from .core import AFSFile, AFSDirectory, AFSListing, AgnosticFileStorage
-from .exceptions import AgnosticFileStorageError
+from .core import (
+    AFSDirectory,
+    AFSFile,
+    AFSListing,
+    AgnosticFileStorage,
+)
+from .errors import (
+    can_not_delete_directory,
+    can_not_delete_file,
+    no_connection,
+    read_error,
+)
 from . import log
 
 
@@ -28,40 +38,28 @@ class SMBFileStorage(AgnosticFileStorage):
     def __init__(self, name, **kwargs):
         super(SMBFileStorage, self).__init__(name)
         self.username = self.get_value(kwargs, 'username')
-        if self.username:
-            self.username = self.username.encode('utf-8')
-        else:
-            raise ValueError('Unknown SMB user for {}'.format(name))
         self.password = self.get_value(kwargs, 'password')
-        if self.password:
-            self.password = self.password.encode('utf-8')
-        else:
-            raise ValueError('Unknown SMB password for {}'.format(name))
-        self.host = self.get_value(kwargs, 'host').encode('utf-8')
+        self.host = self.get_value(kwargs, 'host')
         self.client = socket.gethostname()
-        self.domain = self.get_value(kwargs, 'domain').encode('utf-8')
-        self.service = self.get_value(kwargs, 'service').encode('utf-8')
-        self.fqn = '{}.{}'.format(self.host, self.domain)
-        try:
-            self.server_ip = socket.gethostbyname(self.fqn)
-        except OSError:
-            self.server_ip = None
+        self.domain = self.get_value(kwargs, 'domain')
+        self.service = self.get_value(kwargs, 'service')
         self.is_connected = False
 
-    def __str__(self):
-        bar = '-' * (59 - len(self.name))
-        buff = ['--[ SMBFileStorage {} ]{}'.format(self.name, bar)]
-        buff.append('username: {}'.format(self.username))
-        buff.append('host: {}'.format(self.host))
-        buff.append('client {}'.format(self.client))
-        buff.append('domain: {}'.format(self.domain))
-        buff.append('service: {}'.format(self.service))
-        buff.append('fqn: {}'.format(self.fqn))
-        buff.append('server IP: {}'.format(self.server_ip))
-        buff.append('Current dir: {}'.format(self.current_dir))
-        buff.append('Is Connected: {}'.format(self.is_connected))
-        buff.append('-'*80)
-        return '\n'.join(buff)
+    @property
+    def fqn(self):
+        if self.domain:
+            return '{}.{}'.format(self.host, self.domain)
+        else:
+            return self.host
+
+    def __repr__(self):
+        buff = [f'SMBFileStorage({self.name}']
+        buff.append('username={self.username!repr}')
+        buff.append('host={self.host!repr}')
+        buff.append('host={self.host!repr}')
+        buff.append('domain={self.domain!repr}')
+        buff.append('service={self.service!repr})')
+        return ', '.join(buff)
 
     def open(self):
         self.conn = SMBConnection(
@@ -74,12 +72,14 @@ class SMBFileStorage(AgnosticFileStorage):
             sign_options=SMBConnection.SIGN_NEVER,
             is_direct_tcp=True,
             )
-        self.is_connected = self.conn.connect(self.server_ip, 445)
+        import logging; logging.error("self.fqn is %r (%s)", self.fqn, type(self.fqn))
+        self.is_connected = self.conn.connect(self.fqn, 445)
         super(SMBFileStorage, self).open()
         return self
 
     def close(self):
-        self.conn.close()
+        if self.is_connected:
+            self.conn.close()
         super(SMBFileStorage, self).close()
 
     def is_dir(self, path):
@@ -99,7 +99,7 @@ class SMBFileStorage(AgnosticFileStorage):
             return False
 
     def ls(self):
-        logger.info('Call method ls in SMBFileStorage("%s")', self.name)
+        logger.debug('Call method ls in SMBFileStorage("%s")', self.name)
         tries = 0
         if self.is_connected:
             while tries < NUM_RETRIES:
@@ -112,15 +112,11 @@ class SMBFileStorage(AgnosticFileStorage):
                         else:
                             result.append(AFSFile(f.filename, f.file_size))
                     return result
-                except Exception as err:
-                    pass
-                finally:
+                except Exception:
                     tries += 1
-            raise AgnosticFileStorageError(
-                "Can't read the content of {}".format(self.cwd())
-                )
+            raise read_error(self.cwd())
         else:
-            raise AgnosticFileStorageError('No connection')
+            raise no_connection()
 
     # @log.trace(logger)
     def save(self, filename, stream):
@@ -129,7 +125,7 @@ class SMBFileStorage(AgnosticFileStorage):
             bytes_stored = self.conn.storeFile(self.service, full_path, stream)
             return bytes_stored
         else:
-            raise AgnosticFileStorageError('No connection')
+            raise no_connection()
 
     def mkdir(self, dir_name):
         logger.debug('Calling method mkdir("{}") in SMBFileStorage("{}")'.format(
@@ -144,7 +140,7 @@ class SMBFileStorage(AgnosticFileStorage):
             else:
                 return False
         else:
-            raise AgnosticFileStorageError('No connection')
+            raise no_connection()
 
     def rmdir(self, dir_name):
         logger.debug('Calling method rmdir("{}") in SMBFileStorage("{}")'.format(
@@ -156,10 +152,12 @@ class SMBFileStorage(AgnosticFileStorage):
                 full_path = self.get_absolute_path(dir_name)
                 self.conn.deleteDirectory(self.service, full_path)
                 return True
+            elif self.is_file(dir_name):
+                raise can_not_delete_file(dir_name)
             else:
                 return False
         else:
-            raise AgnosticFileStorageError('No connection')
+            raise no_connection()
 
     def rm(self, file_name):
         logger.debug('Calling method rm("{}") in SMBFileStorage("{}")'.format(
@@ -172,13 +170,8 @@ class SMBFileStorage(AgnosticFileStorage):
                 self.conn.deleteFiles(self.service, full_path)
                 return True
             elif self.is_dir(file_name):
-                raise AgnosticFileStorageError(
-                    "Can't delete a directory with method rm().\n"
-                    "Use the rmdir() method"
-                    )
+                raise can_not_delete_directory(file_name)
             else:
                 return False
         else:
-            raise AgnosticFileStorageError('No connection')
-
-
+            raise no_connection()
